@@ -1,12 +1,15 @@
+from __future__ import annotations
+
 import builtins
 import contextlib
 import math
 import operator
 import warnings
 from collections.abc import Iterable
-from functools import partial
+from functools import partial, reduce
 from itertools import product, repeat
 from numbers import Integral, Number
+from operator import mul
 
 import numpy as np
 from tlz import accumulate, compose, drop, get, partition_all, pluck
@@ -25,6 +28,7 @@ from dask.array.core import (
 )
 from dask.array.creation import arange, diagonal
 from dask.array.dispatch import divide_lookup, nannumel_lookup, numel_lookup
+from dask.array.numpy_compat import NUMPY_GE_200, ComplexWarning
 from dask.array.utils import (
     array_safe,
     asarray_safe,
@@ -383,7 +387,9 @@ def partial_reduce(
     if np.isscalar(meta):
         return Array(graph, name, out_chunks, dtype=dtype)
     else:
-        with contextlib.suppress(AttributeError):
+        with contextlib.suppress(AttributeError), warnings.catch_warnings():
+            if name.startswith("var") or name.startswith("moment"):
+                warnings.simplefilter("ignore", ComplexWarning)
             meta = meta.astype(dtype)
         return Array(graph, name, out_chunks, meta=meta)
 
@@ -747,17 +753,31 @@ def nanmean(a, axis=None, dtype=None, keepdims=False, split_every=None, out=None
 
 
 def moment_chunk(
-    A, order=2, sum=chunk.sum, numel=numel, dtype="f8", computing_meta=False, **kwargs
+    A,
+    order=2,
+    sum=chunk.sum,
+    numel=numel,
+    dtype="f8",
+    computing_meta=False,
+    implicit_complex_dtype=False,
+    **kwargs,
 ):
     if computing_meta:
         return A
     n = numel(A, **kwargs)
 
     n = n.astype(np.int64)
-    total = sum(A, dtype=dtype, **kwargs)
+    if implicit_complex_dtype:
+        total = sum(A, **kwargs)
+    else:
+        total = sum(A, dtype=dtype, **kwargs)
+
     with np.errstate(divide="ignore", invalid="ignore"):
         u = total / n
-    xs = [sum((A - u) ** i, dtype=dtype, **kwargs) for i in range(2, order + 1)]
+    d = A - u
+    if np.issubdtype(A.dtype, np.complexfloating):
+        d = np.abs(d)
+    xs = [sum(d**i, dtype=dtype, **kwargs) for i in range(2, order + 1)]
     M = np.stack(xs, axis=-1)
     return {"total": total, "n": n, "M": M}
 
@@ -785,7 +805,7 @@ def moment_combine(
     if not isinstance(pairs, list):
         pairs = [pairs]
 
-    kwargs["dtype"] = dtype
+    kwargs["dtype"] = None
     kwargs["keepdims"] = True
 
     ns = deepmap(lambda pair: pair["n"], pairs) if not computing_meta else pairs
@@ -801,8 +821,12 @@ def moment_combine(
     total = totals.sum(axis=axis, **kwargs)
 
     with np.errstate(divide="ignore", invalid="ignore"):
-        mu = divide(total, n, dtype=dtype)
-        inner_term = divide(totals, ns, dtype=dtype) - mu
+        if np.issubdtype(total.dtype, np.complexfloating):
+            mu = divide(total, n)
+            inner_term = np.abs(divide(totals, ns) - mu)
+        else:
+            mu = divide(total, n, dtype=dtype)
+            inner_term = divide(totals, ns, dtype=dtype) - mu
 
     xs = [
         _moment_helper(Ms, ns, inner_term, o, sum, axis, kwargs)
@@ -830,6 +854,7 @@ def moment_agg(
     # part of the calculation.
     keepdim_kw = kwargs.copy()
     keepdim_kw["keepdims"] = True
+    keepdim_kw["dtype"] = None
 
     ns = deepmap(lambda pair: pair["n"], pairs) if not computing_meta else pairs
     ns = _concatenate2(ns, axes=axis)
@@ -841,10 +866,13 @@ def moment_agg(
     totals = _concatenate2(deepmap(lambda pair: pair["total"], pairs), axes=axis)
     Ms = _concatenate2(deepmap(lambda pair: pair["M"], pairs), axes=axis)
 
-    mu = divide(totals.sum(axis=axis, **keepdim_kw), n, dtype=dtype)
+    mu = divide(totals.sum(axis=axis, **keepdim_kw), n)
 
     with np.errstate(divide="ignore", invalid="ignore"):
-        inner_term = divide(totals, ns, dtype=dtype) - mu
+        if np.issubdtype(totals.dtype, np.complexfloating):
+            inner_term = np.abs(divide(totals, ns) - mu)
+        else:
+            inner_term = divide(totals, ns, dtype=dtype) - mu
 
     M = _moment_helper(Ms, ns, inner_term, order, sum, axis, kwargs)
 
@@ -917,9 +945,14 @@ def moment(
         dt = dtype
     else:
         dt = getattr(np.var(np.ones(shape=(1,), dtype=a.dtype)), "dtype", object)
+
+    implicit_complex_dtype = dtype is None and np.iscomplexobj(a)
+
     return reduction(
         a,
-        partial(moment_chunk, order=order),
+        partial(
+            moment_chunk, order=order, implicit_complex_dtype=implicit_complex_dtype
+        ),
         partial(moment_agg, order=order, ddof=ddof),
         axis=axis,
         keepdims=keepdims,
@@ -937,9 +970,12 @@ def var(a, axis=None, dtype=None, keepdims=False, ddof=0, split_every=None, out=
         dt = dtype
     else:
         dt = getattr(np.var(np.ones(shape=(1,), dtype=a.dtype)), "dtype", object)
+
+    implicit_complex_dtype = dtype is None and np.iscomplexobj(a)
+
     return reduction(
         a,
-        moment_chunk,
+        partial(moment_chunk, implicit_complex_dtype=implicit_complex_dtype),
         partial(moment_agg, ddof=ddof),
         axis=axis,
         keepdims=keepdims,
@@ -960,9 +996,17 @@ def nanvar(
         dt = dtype
     else:
         dt = getattr(np.var(np.ones(shape=(1,), dtype=a.dtype)), "dtype", object)
+
+    implicit_complex_dtype = dtype is None and np.iscomplexobj(a)
+
     return reduction(
         a,
-        partial(moment_chunk, sum=chunk.nansum, numel=nannumel),
+        partial(
+            moment_chunk,
+            sum=chunk.nansum,
+            numel=nannumel,
+            implicit_complex_dtype=implicit_complex_dtype,
+        ),
         partial(moment_agg, sum=np.nansum, ddof=ddof),
         axis=axis,
         keepdims=keepdims,
@@ -975,10 +1019,9 @@ def nanvar(
 
 
 def _sqrt(a):
-    o = np.sqrt(a)
-    if isinstance(o, np.ma.masked_array) and not o.shape and o.mask.all():
+    if isinstance(a, np.ma.masked_array) and not a.shape and a.mask.all():
         return np.ma.masked
-    return o
+    return np.sqrt(a)
 
 
 def safe_sqrt(a):
@@ -1055,7 +1098,7 @@ def _arg_combine(data, axis, argfunc, keepdims=False):
         arg = arg.ravel()[local_args]
     else:
         local_args = argfunc(vals, axis=axis)
-        inds = np.ogrid[tuple(map(slice, local_args.shape))]
+        inds = list(np.ogrid[tuple(map(slice, local_args.shape))])
         inds.insert(axis, local_args)
         inds = tuple(inds)
         vals = vals[inds]
@@ -1496,7 +1539,8 @@ def cumreduction(
         )
 
     if axis is None:
-        x = x.flatten().rechunk(chunks=x.npartitions)
+        if x.ndim > 1:
+            x = x.flatten().rechunk(chunks=x.npartitions)
         axis = 0
     if dtype is None:
         dtype = getattr(func(np.ones((0,), dtype=x.dtype)), "dtype", object)
@@ -1773,9 +1817,11 @@ def median(a, axis=None, keepdims=False, out=None):
         axis=axis,
         keepdims=keepdims,
         drop_axis=axis if not keepdims else None,
-        chunks=[1 if ax in axis else c for ax, c in enumerate(a.chunks)]
-        if keepdims
-        else None,
+        chunks=(
+            [1 if ax in axis else c for ax, c in enumerate(a.chunks)]
+            if keepdims
+            else None
+        ),
     )
 
     result = handle_out(out, result)
@@ -1808,10 +1854,230 @@ def nanmedian(a, axis=None, keepdims=False, out=None):
         axis=axis,
         keepdims=keepdims,
         drop_axis=axis if not keepdims else None,
-        chunks=[1 if ax in axis else c for ax, c in enumerate(a.chunks)]
-        if keepdims
-        else None,
+        chunks=(
+            [1 if ax in axis else c for ax, c in enumerate(a.chunks)]
+            if keepdims
+            else None
+        ),
     )
 
     result = handle_out(out, result)
     return result
+
+
+@derived_from(np)
+def quantile(
+    a,
+    q,
+    axis=None,
+    out=None,
+    overwrite_input=False,
+    method="linear",
+    keepdims=False,
+    *,
+    weights=None,
+    interpolation=None,
+):
+    """
+    This works by automatically chunking the reduced axes to a single chunk if necessary
+    and then calling ``numpy.quantile`` function across the remaining dimensions
+    """
+    if axis is None:
+        if any(n_blocks > 1 for n_blocks in a.numblocks):
+            raise NotImplementedError(
+                "The da.quantile function only works along an axis.  "
+                "The full algorithm is difficult to do in parallel"
+            )
+        else:
+            axis = tuple(range(len(a.shape)))
+
+    if not isinstance(axis, Iterable):
+        axis = (axis,)
+
+    axis = [ax + a.ndim if ax < 0 else ax for ax in axis]
+
+    # rechunk if reduced axes are not contained in a single chunk
+    if builtins.any(a.numblocks[ax] > 1 for ax in axis):
+        a = a.rechunk({ax: -1 if ax in axis else "auto" for ax in range(a.ndim)})
+
+    if NUMPY_GE_200:
+        kwargs = {"weights": weights}
+    else:
+        kwargs = {}
+
+    result = a.map_blocks(
+        np.quantile,
+        q=q,
+        method=method,
+        interpolation=interpolation,
+        axis=axis,
+        keepdims=keepdims,
+        drop_axis=axis if not keepdims else None,
+        new_axis=0 if isinstance(q, Iterable) else None,
+        chunks=_get_quantile_chunks(a, q, axis, keepdims),
+        **kwargs,
+    )
+
+    result = handle_out(out, result)
+    return result
+
+
+def _span_indexers(a):
+    # We have to create an indexer so that we can index into every quantile slice
+    # The method relies on the quantile axis being the last axis, which means that
+    # we have to calculate reduce(mul, list(a.shape)[:-1]) number of quantiles
+    # We create an indexer combination for each of these quantiles
+
+    shapes = 1 if len(a.shape) <= 2 else reduce(mul, list(a.shape)[1:-1])
+    original_shapes = shapes * a.shape[0]
+    indexers = [tuple(np.repeat(np.arange(a.shape[0]), shapes))]
+
+    for i in range(1, len(a.shape) - 1):
+        indexer = np.repeat(np.arange(a.shape[i]), shapes // a.shape[i])
+        indexers.append(tuple(np.tile(indexer, original_shapes // shapes)))
+        shapes //= a.shape[i]
+    return indexers
+
+
+def _custom_quantile(
+    a,
+    q,
+    axis=None,
+    method="linear",
+    interpolation=None,
+    keepdims=False,
+    **kwargs,
+):
+    if (
+        not {method, interpolation}.issubset({"linear", None})
+        or len(axis) != 1
+        or axis[0] != len(a.shape) - 1
+        or len(a.shape) == 1
+        or a.shape[-1] > 1000
+    ):
+        # bail to nanquantile. Assumptions are pretty strict for now but we
+        # do cover the xarray.quantile case.
+        return np.nanquantile(
+            a,
+            q,
+            axis=axis,
+            method=method,
+            interpolation=interpolation,
+            keepdims=keepdims,
+            **kwargs,
+        )
+    # nanquantile in NumPy is pretty slow if the quantile axis is slow because
+    # each quantile has overhead.
+    # This method works around this by calculating the quantile manually.
+    # Steps:
+    # 1. Sort the array along the quantile axis (this is the most expensive step
+    # 2. Calculate which positions are the quantile positions
+    #    (respecting NaN values, so each quantile can have a different indexer)
+    # 3. Get the neighboring values of the quantile positions
+    # 4. Perform linear interpolation between the neighboring values
+    #
+    # The main advantage is that we get rid of the overhead, removing GIL blockage
+    # and just generally making things faster.
+
+    sorted_arr = np.sort(a, axis=-1)
+    indexers = _span_indexers(a)
+    nr_quantiles = len(indexers[0])
+
+    is_scalar = False
+    if not isinstance(q, Iterable):
+        is_scalar = True
+        q = [q]
+
+    quantiles = []
+    reshape_shapes = (1,) + tuple(sorted_arr.shape[:-1]) + ((1,) if keepdims else ())
+    for single_q in list(q):
+        i = (
+            np.ones(nr_quantiles) * (a.shape[-1] - 1)
+            - np.isnan(sorted_arr).sum(axis=-1).reshape(-1)
+        ) * single_q
+        lower_value, higher_value = np.floor(i).astype(int), np.ceil(i).astype(int)
+
+        # Get neighboring values
+        lower = sorted_arr[tuple(indexers) + (tuple(lower_value),)]
+        higher = sorted_arr[tuple(indexers) + (tuple(higher_value),)]
+
+        # Perform linear interpolation
+        factor_higher = i - lower_value
+        factor_higher = np.where(factor_higher == 0.0, 1.0, factor_higher)
+        factor_lower = higher_value - i
+
+        quantiles.append(
+            (higher * factor_higher + lower * factor_lower).reshape(*reshape_shapes)
+        )
+
+    if is_scalar:
+        return quantiles[0].squeeze(axis=0)
+    else:
+        return np.concatenate(quantiles, axis=0)
+
+
+@derived_from(np)
+def nanquantile(
+    a,
+    q,
+    axis=None,
+    out=None,
+    overwrite_input=False,
+    method="linear",
+    keepdims=False,
+    *,
+    weights=None,
+    interpolation=None,
+):
+    """
+    This works by automatically chunking the reduced axes to a single chunk
+    and then calling ``numpy.nanquantile`` function across the remaining dimensions
+    """
+    if axis is None:
+        if any(n_blocks > 1 for n_blocks in a.numblocks):
+            raise NotImplementedError(
+                "The da.nanquantile function only works along an axis.  "
+                "The full algorithm is difficult to do in parallel"
+            )
+        else:
+            axis = tuple(range(len(a.shape)))
+
+    if not isinstance(axis, Iterable):
+        axis = (axis,)
+
+    axis = [ax + a.ndim if ax < 0 else ax for ax in axis]
+
+    # rechunk if reduced axes are not contained in a single chunk
+    if builtins.any(a.numblocks[ax] > 1 for ax in axis):
+        a = a.rechunk({ax: -1 if ax in axis else "auto" for ax in range(a.ndim)})
+
+    if NUMPY_GE_200:
+        kwargs = {"weights": weights}
+    else:
+        kwargs = {}
+
+    result = a.map_blocks(
+        _custom_quantile,
+        q=q,
+        method=method,
+        interpolation=interpolation,
+        axis=axis,
+        keepdims=keepdims,
+        drop_axis=axis if not keepdims else None,
+        new_axis=0 if isinstance(q, Iterable) else None,
+        chunks=_get_quantile_chunks(a, q, axis, keepdims),
+        **kwargs,
+    )
+
+    result = handle_out(out, result)
+    return result
+
+
+def _get_quantile_chunks(a, q, axis, keepdims):
+    quantile_chunk = [len(q)] if isinstance(q, Iterable) else []
+    if keepdims:
+        return quantile_chunk + [
+            1 if ax in axis else c for ax, c in enumerate(a.chunks)
+        ]
+    else:
+        return quantile_chunk + [c for ax, c in enumerate(a.chunks) if ax not in axis]
