@@ -5,6 +5,8 @@ from itertools import product
 
 import pytest
 
+from dask._task_spec import Task
+
 np = pytest.importorskip("numpy")
 import math
 
@@ -25,6 +27,9 @@ from dask.array.rechunk import (
 from dask.array.utils import assert_eq
 from dask.utils import funcname
 
+if da._array_expr_enabled():
+    pytest.skip("parametrize using unsupported functions", allow_module_level=True)
+
 
 def test_rechunk_internals_1():
     """Test the cumdims_label and _breakpoints and
@@ -545,9 +550,9 @@ def test_dont_concatenate_single_chunks(shape, chunks):
     y = x.rechunk(chunks)
     dsk = dict(y.dask)
     assert not any(
-        funcname(task[0]).startswith("concat")
+        funcname(task.func).startswith("concat")
         for task in dsk.values()
-        if dask.istask(task)
+        if isinstance(task, Task)
     )
 
 
