@@ -8,7 +8,7 @@ import time
 import fsspec
 import pytest
 from fsspec.core import open_files
-from packaging.version import parse as parse_version
+from packaging.version import Version
 
 import dask.bag as db
 from dask.utils import tmpdir
@@ -20,7 +20,7 @@ errs: tuple[type[Exception], ...] = (
     requests.exceptions.RequestException,
     FileNotFoundError,
 )
-if parse_version(fsspec.__version__) > parse_version("0.7.4"):
+if Version(fsspec.__version__) > Version("0.7.4"):
     aiohttp = pytest.importorskip("aiohttp")
     errs = errs + (aiohttp.client_exceptions.ClientResponseError,)
 
@@ -121,7 +121,7 @@ def test_ops_blocksize(dir_server):
         fn = files[1]
         f = open_files(root + fn, block_size=2)[0]
         with f as f:
-            if parse_version(fsspec.__version__) < parse_version("2021.11.1"):
+            if Version(fsspec.__version__) < Version("2021.11.1"):
                 # fails because we want only 12 bytes
                 with pytest.raises(ValueError):
                     assert f.read(10) == expected[:10]
@@ -165,13 +165,16 @@ def test_files(dir_server):
 
 def test_open_glob(dir_server):
     root = "http://localhost:8999/"
-    fs = open_files(root + "/*")
+    fs = open_files(root + "*")
     assert fs[0].path == "http://localhost:8999/a"
     assert fs[1].path == "http://localhost:8999/b"
 
 
 @pytest.mark.network
-@pytest.mark.parametrize("engine", ["pyarrow", "fastparquet"])
+@pytest.mark.parametrize(
+    "engine",
+    ["pyarrow"],
+)
 def test_parquet(engine):
     pytest.importorskip("requests", minversion="2.21.0")
     dd = pytest.importorskip("dask.dataframe")
