@@ -7,6 +7,7 @@ import operator
 import numpy as np
 import pyarrow as pa
 
+from dask import delayed
 from dask._task_spec import DataNode, List, Task
 from dask.dataframe import methods
 from dask.dataframe._pyarrow import to_pyarrow_string
@@ -22,13 +23,10 @@ from dask.dataframe.dask_expr._expr import (
     no_default,
 )
 from dask.dataframe.dask_expr._reductions import Len
-from dask.dataframe.dask_expr._util import (
-    _BackendData,
-    _convert_to_list,
-    _tokenize_deterministic,
-)
+from dask.dataframe.dask_expr._util import _BackendData, _convert_to_list
 from dask.dataframe.dispatch import make_meta
 from dask.dataframe.io.io import _meta_from_array, sorted_division_locations
+from dask.tokenize import _tokenize_deterministic
 from dask.typing import Key
 from dask.utils import funcname, is_series_like
 
@@ -259,10 +257,11 @@ class FromMap(PartitionsFiltered, BlockwiseIO):
     def _meta(self):
         if self.operand("user_meta") is not no_default:
             meta = self.operand("user_meta")
+            return make_meta(meta)
         else:
             vals = [v[0] for v in self.iterables]
-            meta = self.func(*vals, *self.args, **self.kwargs)
-        return make_meta(meta)
+            meta = delayed(self.func)(*vals, *self.args, **self.kwargs)
+            return delayed(make_meta)(meta).compute()
 
     def _divisions(self):
         if self.operand("user_divisions"):
@@ -662,12 +661,21 @@ class FromArray(PartitionsFiltered, BlockwiseIO):
         divisions = divisions + (len(self.frame) - 1,)
         return divisions
 
+    @functools.cached_property
+    def unfiltered_divisions(self):
+        return self._divisions()
+
     def _filtered_task(self, name: Key, index: int) -> Task:
         data = self.frame[slice(index * self.chunksize, (index + 1) * self.chunksize)]
-        if index == len(self.divisions) - 2:
-            idx = range(self.divisions[index], self.divisions[index + 1] + 1)
+        if index == len(self.unfiltered_divisions) - 2:
+            idx = range(
+                self.unfiltered_divisions[index],
+                self.unfiltered_divisions[index + 1] + 1,
+            )
         else:
-            idx = range(self.divisions[index], self.divisions[index + 1])
+            idx = range(
+                self.unfiltered_divisions[index], self.unfiltered_divisions[index + 1]
+            )
 
         if is_series_like(self._meta):
             return Task(
