@@ -11,7 +11,9 @@ import weakref
 from functools import partial
 from operator import add
 
-from distributed import Client, SchedulerPlugin, WorkerPlugin, futures_of, wait
+from packaging.version import Version
+
+from distributed import Client, SchedulerPlugin, futures_of, wait
 from distributed.utils_test import cleanup  # noqa F401
 from distributed.utils_test import client as c  # noqa F401
 from distributed.utils_test import (  # noqa F401
@@ -28,9 +30,7 @@ import dask
 import dask.bag as db
 from dask import compute, delayed, persist
 from dask.base import compute_as_if_collection, get_scheduler
-from dask.blockwise import Blockwise
 from dask.delayed import Delayed
-from dask.layers import ShuffleLayer, SimpleShuffleLayer
 from dask.utils import get_named_args, get_scheduler_lock, tmpdir, tmpfile
 from dask.utils_test import inc
 
@@ -111,10 +111,6 @@ def test_futures_to_delayed_dataframe(c):
     ddf = dd.from_delayed(futures)
     dd.utils.assert_eq(ddf.compute(), pd.concat([df, df], axis=0))
 
-    # Make sure from_delayed is Blockwise
-    if not dd._dask_expr_enabled():
-        assert isinstance(ddf.dask.layers[ddf._name], Blockwise)
-
     with pytest.raises(TypeError):
         ddf = dd.from_delayed([1, 2])
 
@@ -131,8 +127,7 @@ def test_from_delayed_dataframe(c):
     dd.utils.assert_eq(ddf, df, scheduler=c)
 
 
-@pytest.mark.parametrize("fuse", [True, False])
-def test_fused_blockwise_dataframe_merge(c, fuse):
+def test_fused_blockwise_dataframe_merge(c):
     pd = pytest.importorskip("pandas")
     dd = pytest.importorskip("dask.dataframe")
 
@@ -147,12 +142,11 @@ def test_fused_blockwise_dataframe_merge(c, fuse):
     df1 += 10
     df2 += 10
 
-    with dask.config.set({"optimization.fuse.active": fuse}):
-        ddfm = ddf1.merge(ddf2, on=["x"], how="left", shuffle_method="tasks")
-        ddfm.head()  # https://github.com/dask/dask/issues/7178
-        dfm = ddfm.compute().sort_values("x")
-        # We call compute above since `sort_values` is not
-        # supported in `dask.dataframe`
+    ddfm = ddf1.merge(ddf2, on=["x"], how="left", shuffle_method="tasks")
+    ddfm.head()  # https://github.com/dask/dask/issues/7178
+    dfm = ddfm.compute().sort_values("x")
+    # We call compute above since `sort_values` is not
+    # supported in `dask.dataframe`
     dd.utils.assert_eq(
         dfm, df1.merge(df2, on=["x"], how="left").sort_values("x"), check_index=False
     )
@@ -340,10 +334,33 @@ def test_futures_in_graph(c):
     assert xxyy3.compute(scheduler="dask.distributed") == ((1 + 1) + (2 + 2)) + 10
 
 
-def test_zarr_distributed_roundtrip(c):
+@pytest.fixture(scope="function")
+def zarr(c):
+    zarr_lib = pytest.importorskip("zarr")
+    # Zarr-Python 3 lazily allocates a dedicated thread/IO loop
+    # for to execute async tasks. To avoid having this thread
+    # be picked up as a "leaked thread", we manually trigger it's
+    # creation before using zarr
+    try:
+        _ = zarr_lib.core.sync._get_loop()
+        _ = zarr_lib.core.sync._get_executor()
+        yield zarr_lib
+    except AttributeError:
+        yield zarr_lib
+    finally:
+        # Zarr-Python 3 lazily allocates a IO thread, a thread pool executor, and
+        # an IO loop. Here we clean up these resources to avoid leaking threads
+        # In normal operations, this is done as by an atexit handler when Zarr
+        # is shutting down.
+        try:
+            zarr_lib.core.sync.cleanup_resources()
+        except AttributeError:
+            pass
+
+
+def test_zarr_distributed_roundtrip(c, zarr):
     pytest.importorskip("numpy")
     da = pytest.importorskip("dask.array")
-    pytest.importorskip("zarr")
 
     with tmpdir() as d:
         a = da.zeros((3, 3), chunks=(1, 1))
@@ -353,16 +370,18 @@ def test_zarr_distributed_roundtrip(c):
         assert a2.chunks == a.chunks
 
 
-def test_zarr_distributed_with_explicit_directory_store(c):
+def test_zarr_distributed_with_explicit_directory_store(c, zarr):
     pytest.importorskip("numpy")
     da = pytest.importorskip("dask.array")
-    zarr = pytest.importorskip("zarr")
 
     with tmpdir() as d:
         chunks = (1, 1)
         a = da.zeros((3, 3), chunks=chunks)
-        s = zarr.storage.DirectoryStore(d)
-        z = zarr.creation.open_array(
+        if Version(zarr.__version__) < Version("3.0.0.a0"):
+            s = zarr.storage.DirectoryStore(d)
+        else:
+            s = zarr.storage.LocalStore(d, read_only=False)
+        z = zarr.open_array(
             shape=a.shape,
             chunks=chunks,
             dtype=a.dtype,
@@ -375,15 +394,17 @@ def test_zarr_distributed_with_explicit_directory_store(c):
         assert a2.chunks == a.chunks
 
 
-def test_zarr_distributed_with_explicit_memory_store(c):
+def test_zarr_distributed_with_explicit_memory_store(c, zarr):
     pytest.importorskip("numpy")
     da = pytest.importorskip("dask.array")
-    zarr = pytest.importorskip("zarr")
 
     chunks = (1, 1)
     a = da.zeros((3, 3), chunks=chunks)
-    s = zarr.storage.MemoryStore()
-    z = zarr.creation.open_array(
+    if Version(zarr.__version__) < Version("3.0.0.a0"):
+        s = zarr.storage.MemoryStore()
+    else:
+        s = zarr.storage.MemoryStore(read_only=False)
+    z = zarr.open_array(
         shape=a.shape,
         chunks=chunks,
         dtype=a.dtype,
@@ -394,10 +415,9 @@ def test_zarr_distributed_with_explicit_memory_store(c):
         a.to_zarr(z)
 
 
-def test_zarr_in_memory_distributed_err(c):
+def test_zarr_in_memory_distributed_err(c, zarr):
     pytest.importorskip("numpy")
     da = pytest.importorskip("dask.array")
-    zarr = pytest.importorskip("zarr")
 
     chunks = (1, 1)
     a = da.ones((3, 3), chunks=chunks)
@@ -504,10 +524,7 @@ def test_blockwise_array_creation(c, io, fuse):
 @pytest.mark.parametrize(
     "io",
     [
-        "parquet-pyarrow",
-        pytest.param(
-            "parquet-fastparquet", marks=pytest.mark.skip_with_pyarrow_strings
-        ),
+        "parquet",
         "csv",
         # See https://github.com/dask/dask/issues/9793
         pytest.param("hdf", marks=pytest.mark.flaky(reruns=5)),
@@ -518,8 +535,6 @@ def test_blockwise_array_creation(c, io, fuse):
 def test_blockwise_dataframe_io(c, tmpdir, io, fuse, from_futures):
     pd = pytest.importorskip("pandas")
     dd = pytest.importorskip("dask.dataframe")
-    if dd._dask_expr_enabled():
-        pytest.xfail("doesn't work yet")
 
     df = pd.DataFrame({"x": [1, 2, 3] * 5, "y": range(15)})
 
@@ -530,15 +545,10 @@ def test_blockwise_dataframe_io(c, tmpdir, io, fuse, from_futures):
     else:
         ddf0 = dd.from_pandas(df, npartitions=3)
 
-    if io == "parquet-pyarrow":
+    if io == "parquet":
         pytest.importorskip("pyarrow")
         ddf0.to_parquet(str(tmpdir))
         ddf = dd.read_parquet(str(tmpdir))
-    elif io == "parquet-fastparquet":
-        pytest.importorskip("fastparquet")
-        with pytest.warns(FutureWarning):
-            ddf0.to_parquet(str(tmpdir), engine="fastparquet")
-            ddf = dd.read_parquet(str(tmpdir), engine="fastparquet")
     elif io == "csv":
         ddf0.to_csv(str(tmpdir), index=False)
         ddf = dd.read_csv(os.path.join(str(tmpdir), "*"))
@@ -552,14 +562,6 @@ def test_blockwise_dataframe_io(c, tmpdir, io, fuse, from_futures):
 
     df = df[["x"]] + 10
     ddf = ddf[["x"]] + 10
-    if not dd._dask_expr_enabled():
-        with dask.config.set({"optimization.fuse.active": fuse}):
-            ddf.compute()
-            dsk = dask.dataframe.optimize(ddf.dask, ddf.__dask_keys__())
-            # dsk should not be a dict unless fuse is explicitly True
-            assert isinstance(dsk, dict) == bool(fuse)
-
-            dd.assert_eq(ddf, df, check_index=False)
 
 
 def test_blockwise_fusion_after_compute(c):
@@ -731,55 +733,7 @@ async def test_futures_in_subgraphs(c, s, a, b):
 
     ddf = ddf[ddf.uid.isin(range(29))].persist()
     ddf["day"] = ddf.enter_time.dt.day_name()
-    ddf = await c.submit(dd.categorical.categorize, ddf, columns=["day"], index=False)
-
-
-@pytest.mark.parametrize(
-    "max_branch, expected_layer_type",
-    [
-        (32, SimpleShuffleLayer),
-        (2, ShuffleLayer),
-    ],
-)
-@gen_cluster(client=True, nthreads=[("", 1)] * 2)
-async def test_shuffle_priority(c, s, a, b, max_branch, expected_layer_type):
-    pd = pytest.importorskip("pandas")
-    dd = pytest.importorskip("dask.dataframe")
-    if dd._dask_expr_enabled():
-        pytest.skip("Checking layers doesn't make sense")
-
-    class EnsureSplitsRunImmediatelyPlugin(WorkerPlugin):
-        failure = False
-
-        def setup(self, worker):
-            self.worker = worker
-
-        def transition(self, key, start, finish, **kwargs):
-            if finish == "executing" and not all(
-                "split" in ts.key for ts in self.worker.state.executing
-            ):
-                if any("split" in ts.key for ts in list(self.worker.state.ready)):
-                    EnsureSplitsRunImmediatelyPlugin.failure = True
-                    raise RuntimeError("Split tasks are not prioritized")
-
-    await c.register_plugin(EnsureSplitsRunImmediatelyPlugin())
-
-    # Test marked as "flaky" since the scheduling behavior
-    # is not deterministic. Note that the test is still
-    # very likely to fail every time if the "split" tasks
-    # are not prioritized correctly
-
-    df = pd.DataFrame({"a": range(1000)})
-    ddf = dd.from_pandas(df, npartitions=10)
-
-    ddf2 = ddf.shuffle("a", shuffle_method="tasks", max_branch=max_branch)
-
-    shuffle_layers = set(ddf2.dask.layers) - set(ddf.dask.layers)
-    for layer_name in shuffle_layers:
-        if "shuffle" in layer_name:
-            assert isinstance(ddf2.dask.layers[layer_name], expected_layer_type)
-    await c.compute(ddf2)
-    assert not EnsureSplitsRunImmediatelyPlugin.failure
+    ddf = await c.submit(ddf.categorize, columns=["day"], index=False)
 
 
 @gen_cluster(client=True)
@@ -788,10 +742,9 @@ async def test_map_partitions_da_input(c, s, a, b):
     np = pytest.importorskip("numpy")
     pd = pytest.importorskip("pandas")
     da = pytest.importorskip("dask.array")
-    dd = pytest.importorskip("dask.dataframe")
+    pytest.importorskip("dask.dataframe")
     datasets = pytest.importorskip("dask.datasets")
-    if dd._dask_expr_enabled():
-        pytest.skip("roundtripping through arrays doesn't work yet")
+    pytest.xfail("roundtripping through arrays doesn't work yet")
 
     def f(d, a):
         assert isinstance(d, pd.DataFrame)
@@ -810,8 +763,7 @@ def test_map_partitions_df_input():
     """
     pd = pytest.importorskip("pandas")
     dd = pytest.importorskip("dask.dataframe")
-    if dd._dask_expr_enabled():
-        pytest.skip("map partitions can't deal with delayed properly")
+    pytest.xfail("map partitions can't deal with delayed properly")
 
     def f(d, a):
         assert isinstance(d, pd.DataFrame)
@@ -880,8 +832,6 @@ async def test_non_recursive_df_reduce(c, s, a, b):
     # See https://github.com/dask/dask/issues/8773
     pd = pytest.importorskip("pandas")
     dd = pytest.importorskip("dask.dataframe")
-    if dd._dask_expr_enabled():
-        pytest.skip("we don't offer a public reduction")
 
     class SomeObject:
         def __init__(self, val):
@@ -1108,3 +1058,16 @@ async def test_release_persisted_futures_without_gc(c, s, a, b):
             await asyncio.sleep(0.01)
     finally:
         gc.enable()
+
+
+@gen_cluster(client=True)
+async def test_fusion_barrier_task(c, s, a, b):
+    np = pytest.importorskip("numpy")
+    da = pytest.importorskip("dask.array")
+
+    a = np.random.default_rng().uniform(0, 1, 100).reshape((10, 10))
+    x = da.from_array(a, chunks=(10, 1))
+    new = ((1,) * 10, (10,))
+    x2 = da.rechunk(x, chunks=new, method="p2p")
+    result = await c.compute(x2)
+    da.assert_eq(result, a)
